function ProPixxMovie(MovieFile, Duration)
    % Press any key to start movie once ready-screen is up.
    % Press escape at any time to quit.
    % Space during playback will pause the movie, press any key to resume, but
    % time keeps running and movie will stop at requested duration even if
    % paused.
    %
    % Movie file can have full path, or will be assumed in the same directory as
    % this function.
    %
    % It is recommended to only have one screen enabled for PsychToolbox to
    % function properly, however this may work fine in a dual screen setup
    % configured properly (e.g. same refresh rate, and after passing required
    % PTB tests once). As should always be the case with PTB, the primary screen
    % will be used.
    %
    % Marc Lalancette, The Neuro, 2024-12-07

    % ----------------------------------------------------------------------
    % User parameters

    % Use ProPixx video mode C48 for accurate color rendering.
    MovieVidMode = 3; % Set video mode to C48 for high-bit-depth color 3: C48, 0: C24 (default)
    WaitText = 'Le film commencera bientôt. \nNe bougez pas.\n\nThe movie will start soon. \nStay still.';
    EndText = 'Fin\n\nEnd';
    FontSize = 48;
    % Option to hide mouse cursor
    isHideCursor = true;
    % Option to send event markers, to potentially start MEG recording in sync.
    isUseParallelPort = true;
    PptAddress = hex2dec('2000'); % On Windows: Device Manager > Ports > (your port name) > Properties > Resources > I/O Range (first number of first one), displayed in hexadecimal

    % ----------------------------------------------------------------------
    % Movie to play, full path if not in same folder as this function
    if nargin < 1 || isempty(MovieFile)
        error('Please provide movie file to play.')
        % MovieFile = 'PartlyCloudy_Stereo_CFR_fixed.mp4';
    end
    if nargin < 2 || isempty(Duration)
        Duration = 24*3600; % 24 hours, i.e. until the end of the movie
    end

    % Mode 2 has best performance.
    RenderMode = 2; % 0: No processing whatsoever, 1: PsychImaging pipeline, 2: pixelFormat & Screen('HookFunction')
    Gamma = 2.4; % Used in render mode 1
    % Print additional information to command window
    isDebug = false;
    % If no processing: use default video mode
    if RenderMode == 0 && ~isDebug
        MovieVidMode = 0;
    end

    % Screen configuration check
    iScr = Screen('Screens');
    if numel(iScr) == 1
        screenNumber = 0; 
    else
        % This won't work with dual screens if PTB tests not passed.
        % ProPixx should be set as Window's primary display
        screenNumber = 1;
        warning('Multi-screen setup detected. May affect rendering.');
    end

    % Check the movie
    if isempty(fileparts(MovieFile))
        scriptDir = fileparts(mfilename('fullpath')); 
        MovieFile = fullfile(scriptDir, MovieFile);
    end
    if ~exist(MovieFile, 'file') 
        error('Image file not found: %s', MovieFile);
    end

    % Prepare parallel port
    if isUseParallelPort
        ioObj = io64; %create a parallel port handle
        if io64(ioObj)  % returns 0 if the port driver is loaded & ready
            error('Unable to initialize parallel port. Set isUsePpt=false if no port available.')
        end
        % Set to zero (and example usage):
        % io64(ioObj, PptAddress, 255);
        % WaitSecs(0.001);
        io64(ioObj, PptAddress, 0);
    end

    % Color space conversion, BT.709 to ProPixx
    % This gives the same result as my calculations with Y=1.
    sRGB = [0.64, 0.33; 0.3, 0.6; 0.15, 0.06; 0.3127, 0.329]';
    ProPixx = [0.665, 0.321; 0.172, 0.726; 0.163, 0.039; 0.3127, 0.329]';
    ColorConv = eye(4);
    ColorConv(1:3, 1:3) = ConvertRGBSourceToRGBTargetColorSpace(sRGB, ProPixx);

    Screen('Preference', 'DefaultFontSize', FontSize);
    Screen('Preference', 'TextAntiAliasing', 2); 
    Screen('Preference', 'DefaultFontStyle', 0); % 1 = bold, 0 = normal

    try
        % Raise priority for stimulus presentation
        % Priority(1); 

        % Initialize Datapixx
        Datapixx('Open'); % Open the Datapixx connection
        % Store initial settings to restore after the movie
        DpStatus = Datapixx('GetVideoStatus');
        initPixMode = DpStatus.pixelMode; % Check pixel mode status
        initVidMode = DpStatus.mode; % Get current video mode
        fprintf('Initial Pixel Mode: %d, Video Mode: %d\n', initPixMode, initVidMode);
        % Enable full pixel mode and set the video mode
        Datapixx('EnablePixelMode', 0); % Full pixel mode to track movie progression in MEG GPIO channel.
        Datapixx('SetVideoMode', MovieVidMode); % Set to C48 video mode
        Datapixx('RegWrRd'); % Write and read back the register cache to apply changes
        % Verify that the settings have been applied
        DpStatus = Datapixx('GetVideoStatus');
        if ~DpStatus.pixelMode
            error('Unable to enable pixel mode.');
        end
        if DpStatus.mode ~= MovieVidMode
            error('Unable to set C48 video mode.');
        end
        Datapixx('Close');

        % For debugging or getting more info on processing pipeline.
        if isDebug
            Screen('Preference', 'Verbosity', 4);
        end

        % Open a fullscreen window
        if RenderMode > 0 || MovieVidMode == 3
            PsychImaging('PrepareConfiguration'); 
            if RenderMode == 1
                PsychImaging('AddTask', 'FinalFormatting', 'DisplayColorCorrection', 'SimpleGamma');
                PsychImaging('AddTask', 'FinalFormatting', 'DisplayColorCorrection', 'MatrixMultiply4');
            end
            if MovieVidMode == 3 % C48 high bit depth mode
                PsychImaging('AddTask', 'General', 'EnableDataPixxC48Output', 2); % mode 2: average even-odd pixels.
            else
                % Still use high bit depth for intermediate computations?
                PsychImaging('AddTask', 'General', 'FloatingPoint32Bit');
            end
            win = PsychImaging('OpenWindow', screenNumber, 0); % Black background
            if RenderMode == 1
                PsychColorCorrection('SetEncodingGamma', win, Gamma)
                PsychColorCorrection('SetMultMatrix4', win, ColorConv)
                PsychColorCorrection('SetColorClampingRange', win, 0.0, 1.0)
            end
            % Overwrite PTB default of hiding pixel sync raster line.
            if MovieVidMode == 3 % C48 high bit depth mode
                Datapixx('SetVideoPixelSyncLine', 0, 1, 0); % line 0, single line, do not blank line
            end
            if RenderMode == 2
                Screen('HookFunction', win, 'WindowColorGamut', [], ProPixx);
            end
        else % RenderMode == 0, no processing
            win = Screen('OpenWindow', screenNumber, 0); % Black background
        end

        % Hide mouse cursor
        if isHideCursor
            HideCursor(win);
        end

        KbName('UnifyKeyNames');
        escapeKey = KbName('ESCAPE');
        spaceKey = KbName('SPACE');
        % This seems necessary as other tasks may change it and it stays even
        % after computer restart??
        RestrictKeysForKbCheck([]);

        % Open movie file
        if RenderMode == 2
            % pixelFormat=11: 16 bpc RGB, 32 bit float buffer
            % maxNumberThreads=0 (auto)
            % Unclear if we should use gamma 2.4, but not available in PTB (or GStreamer it seems).
            % So here we override to use gamma 2.2 instead of inverse BT.709 OETF (which has linear section near zero, i.e. dark).
            [movie, ~, fps, ~, ~, cnt, ~, movieMetaData] = Screen('OpenMovie', win, MovieFile, [], [], [], 11, 0, 'OverrideEOTF=4'); % 4=GST_VIDEO_TRANSFER_GAMMA22 1=GST_VIDEO_TRANSFER_GAMMA10 (linear for test)
        else
            [movie, ~, fps, ~, ~, cnt, ~, movieMetaData] = Screen('OpenMovie', win, MovieFile, [], [], [], [], 0);
        end

        % Start playback engine
        Screen('PlayMovie', movie, 1);

        % Debugging info
        if isDebug
            disp(movieMetaData);
            fprintf('\n\n====== Screen hook dump:\n');
            Screen('HookFunction', win, 'DumpAll');
            fprintf('\n====== End screen hook dump.\n\n');
        end

        % Ready screen
        % For nice looking anti-aliased text, need an FBO when using C48 mode.
        if MovieVidMode == 3
            % Create an offscreen window
            textWin = Screen('OpenOffscreenWindow', win, [0, 0, 0]);
            % Draw text on the offscreen window
            DrawFormattedText(textWin, WaitText, 'center', 'center', 255*[1 1 1]);
            % Convert the text to a texture
            textTexture = Screen('MakeTexture', win, Screen('GetImage', textWin));
            % Copy the texture to the main window
            Screen('DrawTexture', win, textTexture);
        else
            DrawFormattedText(win, WaitText, 'center', 'center', 255*[1 1 1]);
        end
        Screen('Flip', win);

        % Wait for keypress to start
        % Wait until user releases keys on keyboard
        KbReleaseWait;
        KbWait;
        [keyIsDown, ~, keyCode] = KbCheck; % allow quitting
        if keyIsDown && keyCode(escapeKey)
            EndExpCleanup;
            return;
        end
        % Wait until user releases keys on keyboard
        KbReleaseWait;
        if isDebug
            fprintf('Starting movie loop.\n');
        end

        % Playback loop: Runs until end of movie, required duration, or keypress
        isStarted = false;
        StartTime = GetSecs();
        while GetSecs() < StartTime + Duration
            % Wait for next movie frame, retrieve texture handle to it
            tex = Screen('GetMovieImage', win, movie);
            % A negative value means end of movie reached
            if tex <= 0
                break;
            end
            % Draw the new texture immediately to screen
            Screen('DrawTexture', win, tex);

            % Update display, measure starting time on first frame
            if ~isStarted
                % Overlay white upper-left pixel to mark movie start
                Screen('FillRect', win, [1, 1, 1], [0, 0, 1, 1]); % [R,G,B], [L,T,R,B]
                % Send trigger on parallel port to mark movie start
                if isUseParallelPort
                    io64(ioObj, PptAddress, 255);
                end
                StartTime = Screen('Flip', win);
                fprintf('First frame drawn and flipped.\n');
                isStarted = true;
                if isUseParallelPort
                    % No real need to wait, MEG detects at 192kHz
                    WaitSecs(0.0001);
                    io64(ioObj, PptAddress, 0);
                end
            else
                Screen('Flip', win);
            end

            % Release texture
            Screen('Close', tex); % Important, otherwise playback slow (mem leak?)

            % Check for key press to exit
            [keyIsDown, ~, keyCode] = KbCheck;
            if keyIsDown 
                if keyCode(escapeKey)
                    break;
                elseif keyCode(spaceKey)
                    Screen('PlayMovie', movie, 0);
                    % Wait for keypress to resume
                    KbReleaseWait; % wait for pause key to be released before checking again
                    while ~KbCheck
                        WaitSecs(0.05);
                    end
                    [keyIsDown, ~, keyCode] = KbCheck; % allow escaping during pause.
                    if keyIsDown && keyCode(escapeKey)
                        break;
                    end
                    KbReleaseWait; % wait for release to resume
                    Screen('PlayMovie', movie, 1);
                end
            end
        end

        % Stop playback
        Screen('PlayMovie', movie, 0);

        % Close movie
        Screen('CloseMovie', movie);

        % End screen
        % Draw text on the offscreen window
        Screen('FillRect', textWin, [0, 0, 0]); % Clear previous text.        
        DrawFormattedText(textWin, EndText, 'center', 'center', 255*[1 1 1]);
        % Convert the text to a texture
        textTexture = Screen('MakeTexture', win, Screen('GetImage', textWin));
        % Copy the texture to the main window
        Screen('DrawTexture', win, textTexture);
        Screen('Flip', win);
        % Close the offscreen window after use (at the end)
        Screen('Close', textWin);
        
        % Wait for keypress to exit
        while ~KbCheck
            WaitSecs(0.05);
        end
        KbReleaseWait;

        EndExpCleanup;

    catch %#ok<CTCH>
        EndExpCleanup(true);
        return;
    end

    function EndExpCleanup(isError)
        if nargin < 1 || isempty(isError)
            isError = false;
        end

        if isHideCursor && exist('win', 'var')
            ShowCursor(win);
        end
        % Close Screen
        sca;

        % Reset projector settings: pixel mode and video mode
        fprintf('Resetting ProPixx.\n')
        Datapixx('Open');
        Datapixx('SetVideoMode', initVidMode); % Restore initial video mode
        DpStatus = Datapixx('GetVideoStatus');
        if DpStatus.pixelMode ~= initPixMode
            if initPixMode
                % Looks like we can't check for full vs GB modes.
                Datapixx('EnablePixelMode');
            else
                Datapixx('DisablePixelMode');
            end
        end
        Datapixx('RegWrRd'); % Apply changes
        Datapixx('Close');

        % Reset to normal priority, though probably not needed.
        Priority(0);

        if ~isError
            fprintf('Task ended gracefully.\n');
        else
            psychrethrow(psychlasterror);
        end
    end

end

